<?php

if (!defined('ABSPATH')) {
	exit; // Exit if accessed directly
}

if (!class_exists('Rigid_Transfer_Content')) {

	/**
	 * Singelton class to manage import/export functionality
	 *
	 * @author aatanasov
	 */
	class Rigid_Transfer_Content {

		/**
		 * Current WCFM options
		 *
		 * @var String
		 */
		private $wcfm_options;

		/**
		 * Current theme options
		 *
		 * @var String
		 */
		private $theme_options;

		/**
		 * Location of demo files
		 *
		 * @var String
		 */
		private $demo_location;

		/**
		 * Export file name
		 *
		 * @var String
		 */
		public $export_filename;

		/**
		 * Delimiter for separating different settings in the file
		 *
		 * @var String
		 */
		private $delimiter;

		/**
		 * Returns the *Rigid_Transfer_Content* instance of this class.
		 *
		 * @staticvar Singleton $instance The *Rigid_Transfer_Content* instances of this class.
		 *
		 * @return Rigid_Transfer_Content The *Rigid_Transfer_Content* instance.
		 */
		public static function getInstance() {
			static $instance = null;
			if ($instance === null) {
				$instance = new Rigid_Transfer_Content();
			}

			return $instance;
		}

		/**
		 * Protected constructor to prevent creating a new instance of the
		 * *Rigid_Transfer_Content* via the `new` operator from outside of this class.
		 */
		protected function __construct() {

			$this->setDelimiter('|||');
			$this->export_filename = get_template_directory() . '/store/settings/' . get_bloginfo('name') . '_settings_' . date('Y_m_d') . '.txt';
			$this->demo_location = get_template_directory() . '/store/demo/';

			global $wp_filesystem;

			if (empty($wp_filesystem)) {
				require_once (ABSPATH . '/wp-admin/includes/file.php');
				WP_Filesystem();
			}
		}

		/**
		 * Private clone method to prevent cloning of the instance of the
		 * *Rigid_Transfer_Content* instance.
		 *
		 * @return void
		 */
		private function __clone() {

		}

		/**
		 * Private unserialize method to prevent unserializing of the *Rigid_Transfer_Content*
		 * instance.
		 *
		 * @return void
		 */
		public function __wakeup() {

		}

		/**
		 * Set delimiter
		 *
		 * @return String
		 */
		public function getDelimiter() {
			return $this->delimiter;
		}

		/**
		 * Get delimiter
		 *
		 * @param String $delimiter
		 */
		public function setDelimiter($delimiter) {
			$this->delimiter = $delimiter;
		}

		/**
		 * Get demo location
		 *
		 * @return String
		 */
		public function getDemoLocation() {
			return $this->demo_location;
		}

		/**
		 * Gets the theme name from the stylesheet (lowercase and without spaces)
		 *
		 * @return String
		 */
		private function getCurrentThemeDirToLower() {
			// This gets the theme name from the stylesheet (lowercase and without spaces)
			$themename_orig = get_option('stylesheet');
			$themename = preg_replace("/\W/", "_", strtolower($themename_orig));

			if(!get_option($themename)) {
				// remove the '_child' if there is
				$themename = substr($themename, 0,-6);
			}

			return $themename;
		}

		private function get_attachment_id_by_slug( $slug ) {
			$args    = array(
				'post_type'      => 'attachment',
				'name'           => sanitize_title( $slug ),
				'posts_per_page' => 1,
				'post_status'    => 'inherit',
			);
			$_header = get_posts( $args );
			$header  = $_header ? array_pop( $_header ) : null;

			return $header ? $header->ID : '';
		}

		/**
		 * Get theme options
		 *
		 * @return String theme options
		 */
		private function getThemeOptions() {
			if (!$this->theme_options) {
				$this->theme_options = get_option($this->getCurrentThemeDirToLower());
			}

			return $this->theme_options;
		}

		/**
		 * Get WCFM options
		 *
		 * @return String WCFM options
		 */
		private function getWCFMOptions() {
			if ( ! $this->wcfm_options ) {
				$wcfmmp_marketplace_options = get_option( 'wcfm_marketplace_options', array() );
				$wcfm_store_color_settings  = get_option( 'wcfm_store_color_settings', array() );
				$wcfm_membership_options    = get_option( 'wcfm_membership_options', array() );

				$this->wcfm_options = array(
					'wcfmmp_marketplace_options' => $wcfmmp_marketplace_options,
					'wcfm_store_color_settings'  => $wcfm_store_color_settings,
					'wcfm_membership_options'    => $wcfm_membership_options,
				);
			}

			return $this->wcfm_options;
		}

		/**
		 * Encodes the options and stores the export.
		 *
		 * @return int Result
		 */
		protected function getEncodedOptions() {

			$encodedOptions = json_encode($this->getThemeOptions());

			return $encodedOptions;
		}

		protected function getEncodedWCFMOptions() {
			return json_encode( $this->getWCFMOptions() );
		}

		/**
		 * Stores given file.
		 *
		 * @param String $filename Filename of the export file
		 * @param String $data Data to be stored
		 * @return int Result
		 */
		protected function storeFile($filename, $data) {

			/**
			 * @global WP_Filesystem_Base $wp_filesystem subclass
			 */
			global $wp_filesystem;

			return $wp_filesystem->put_contents($filename, $data);
		}

		/**
		 * Decodes settings during Import
		 *
		 * @param String $encoded_settings Encoded settings
		 * @return mixed
		 */
		protected function decodeSettings($encoded_settings) {
			return json_decode($encoded_settings, TRUE);
		}

		/**
		 * Imports encoded options
		 *
		 * @param String $encoded_options Encoded options
		 */
		protected function importOptions($encoded_options) {
			$options = $this->decodeSettings($encoded_options);

			update_option($this->getCurrentThemeDirToLower(), $options);
		}

		/**
		 * Import WP content
		 */
		public function importWPContent($file) {

			global $wpdb;

			add_filter("http_request_args", array(&$this, "setHttpRequestTimeout"), 10, 1);

			if (class_exists('RigidImport')) {

				$wp_import = new RigidImport();
				$wp_import->fetch_attachments = true;
				$wp_import->import($file);
			}

		}

		public function setHttpRequestTimeout($req) {
			$req["timeout"] = 360;
			return $req;
		}

		protected function getEncodedWidgets() {

			$sidebars_array = get_option('sidebars_widgets');
			$widget_types = array();
			$all_widgets_options = array();

			// get all registered widget types
			foreach ($sidebars_array as $sidebar_name => $widgets) {
				if ('wp_inactive_widgets' !== $sidebar_name && is_array($widgets)) {
					foreach ($widgets as $widget_index) {
						$widget_types[] = trim(substr($widget_index, 0, strrpos($widget_index, '-')));
					}
				}
			}

			// remove duplicates
			array_unique($widget_types);

			// get widget values for each type
			foreach ($widget_types as $widget_type) {
				$all_widgets_options['widget_' . $widget_type] = get_option('widget_' . $widget_type);
			}

			$sidebars_and_widgets = array('sidebars' => $sidebars_array, 'widgets' => $all_widgets_options);
			$encodedWidgets = json_encode($sidebars_and_widgets);

			return $encodedWidgets;
		}

		public function exportSettings() {

			$encodedOptions = $this->getEncodedOptions();
			$encodedWidgets = $this->getEncodedWidgets();

			$upload_dir = wp_upload_dir();
				$file_name = 'rigid_theme_options_' .  get_bloginfo('name') . '_' . date('Y_m_d') . '.txt';
			$file_path = $upload_dir['path'] . '/'.$file_name;

			return $this->storeFile($file_path, $encodedOptions . $this->getDelimiter() . $encodedWidgets);
		}

		public function exportWCFMSettings() {
			$encodedWCFMOptions = $this->getEncodedWCFMOptions();

			$upload_dir = wp_upload_dir();
			$file_name = 'rigid_wcfm_options_' .  get_bloginfo('name') . '_' . date('Y_m_d') . '.txt';
			$file_path = $upload_dir['path'] . '/'.$file_name;

			return $this->storeFile($file_path, $encodedWCFMOptions);
		}

		public function exportThemeOptions() {

			$encodedOptions = $this->getEncodedOptions();
			$upload_dir = wp_upload_dir();
			$file_name = 'rigid_theme_options_' . date('Y_m_d') . '.txt';
			$file_path = $upload_dir['path'] . '/'.$file_name;

			if($this->storeFile($file_path, $encodedOptions)) {
				return $file_path;
			}

			return false;
		}

		public function importSettings($filename, $widget_menu_1, $widget_menu_2, $widget_menu_3, $widget_menu_4, $widget_menu_5, $only_options = false) {
			/**
			 * @global WP_Filesystem_Base $wp_filesystem subclass
			 */
			global $wp_filesystem;

			$file_error = false;
			$data = $wp_filesystem->get_contents($filename);

			if ($data) {
				$settings_array = explode($this->getDelimiter(), $data);
				$sidebars_and_widgets = '';

				if ( is_array( $settings_array ) && ! empty( $settings_array ) ) {
					$options              = $this->decodeSettings( $settings_array[0] );

					if(array_key_exists(1, $settings_array)) {
						$sidebars_and_widgets = $this->decodeSettings( $settings_array[1] );
					}

					// Keep envato_username and envato_api_key unchanged after the import
					$options['envato_username'] = rigid_get_option('envato_username');
					$options['envato_api_key'] = rigid_get_option('envato_api_key');

					update_option( $this->getCurrentThemeDirToLower(), $options );
					if($sidebars_and_widgets && !$only_options) {
						update_option( 'sidebars_widgets', $sidebars_and_widgets['sidebars'] );

						foreach ( $sidebars_and_widgets['widgets'] as $widget_option_name => $widget_options ) {

							if ( $widget_option_name == 'widget_nav_menu' ) {
								foreach ( $widget_options as $key => $option ) {
									if ( is_array( $option ) ) {
										if ( isset($option['title']) && strcasecmp( $option['title'], 'Information' ) == 0 && $widget_menu_1 ) {
											$widget_options[ $key ]['nav_menu'] = $widget_menu_1->term_id;
										} elseif ( isset($option['title']) && strcasecmp( $option['title'], 'Extras' ) == 0 && $widget_menu_2 ) {
											$widget_options[ $key ]['nav_menu'] = $widget_menu_2->term_id;
										} elseif ( isset($option['title']) && strcasecmp( $option['title'], 'The Shop' ) == 0 && $widget_menu_3 ) {
											$widget_options[ $key ]['nav_menu'] = $widget_menu_3->term_id;
										} elseif (isset($option['nav_menu']) && $option['nav_menu'] === 89){
											$widget_options[ $key ]['nav_menu'] = $widget_menu_4->term_id;
										} elseif (isset($option['nav_menu']) && $option['nav_menu'] === 98){
											$widget_options[ $key ]['nav_menu'] = $widget_menu_5->term_id;
										}
									}
								}

							}
							update_option( $widget_option_name, $widget_options );
						}
					}
				} else {
					$file_error = true;
				}
			}

			if ($file_error) {
				return new WP_Error('settings_import_file_error', esc_html__('There was error with settings file.', 'rigid'));
			}
		}

		public function importWCFMSettings($filename) {
			/**
			 * @global WP_Filesystem_Base $wp_filesystem subclass
			 */
			global $wp_filesystem;

			$file_error = false;
			$data = $wp_filesystem->get_contents($filename);

			if ($data) {
				$settings_array = $this->decodeSettings( $data );
				if ( is_array( $settings_array ) && ! empty( $settings_array ) ) {
					if(!empty($settings_array['wcfm_marketplace_options'])) {
						update_option( 'wcfm_marketplace_options', $settings_array['wcfm_marketplace_options'] );
					}
					if(!empty($settings_array['wcfm_store_color_settings'])) {
						update_option( 'wcfm_store_color_settings', $settings_array['wcfm_store_color_settings'] );
					}
					if(!empty($settings_array['wcfm_membership_options'])) {
						update_option( 'wcfm_membership_options', $settings_array['wcfm_membership_options'] );
					}
				} else {
					$file_error = true;
				}
			}

			if ($file_error) {
				return new WP_Error('settings_import_file_error', esc_html__('There was error with settings file.', 'rigid'));
			}
		}

		/**
		 * Import all revolution sliders
		 *
		 * @param string $demo_name
		 * @return boolean
		 */
		public function importRevSliders($demo_name = 'one') {
			if (!class_exists('RevSliderSliderImport')) {
				return false;
			}

			$rev_directory = $this->getDemoLocation() . $demo_name . '/revsliders/';
			foreach ( glob( $rev_directory . '*.zip' ) as $filename ) { // get all files from revsliders data dir
				$filename    = basename( $filename );
				$rev_files[] = $rev_directory . $filename;
			}

			if ( ! isset( $rev_files ) || ! is_array( $rev_files ) ) {
				return false;
			}

			$revSliderSliderImport = new RevSliderSliderImport();

			$is_template = false;
			$single_slide = true;
			$update_animation = true;
			$update_navigation = true;
			$install = true;
			foreach ($rev_files as $rev_file) { // finally import rev slider data files
				$importSliderResult = $revSliderSliderImport->import_slider( $update_animation, $rev_file, $is_template, $single_slide, $update_navigation, $install );
			}
		}

		/**
		 * @param string $demo_name
		 *
		 * @return bool
		 */
		public function doImportDemo($demo_name = 'one') {

			echo 'import started ' . date(DATE_RFC2822) . '<br/>';

			// Delete default Shop page created by WooCommerce to be able to import the one from the demo.
			// Otherwise, page with slug shop-2 will be created, which will mess up the links
			$default_shop_page = get_page_by_title( 'Shop' );
			if ( $default_shop_page instanceof WP_Post ) {
				wp_delete_post( $default_shop_page->ID, true );
			}

			// Delete default Vendor Registration page created by WC Mar to be able to import the one from the demo.
			if(class_exists('MVX')) {
				$default_vendor_registration_page = get_page_by_title( 'Vendor Registration' );
				if ( $default_vendor_registration_page instanceof WP_Post ) {
					wp_delete_post( $default_vendor_registration_page->ID, true );
				}
			}

			// Delete current menus
			$all_pages_menu_for_del = wp_get_nav_menu_object('Main menu');
			$top_menu_for_del = wp_get_nav_menu_object('Top menu');
			$footer_menu_for_del = wp_get_nav_menu_object('Footer Menu');

			$widget_menu_1_for_del =  wp_get_nav_menu_object('Information');
			$widget_menu_2_for_del =  wp_get_nav_menu_object('Extras');
			$widget_menu_3_for_del =  wp_get_nav_menu_object('The Shop');
			$widget_menu_4_for_del = wp_get_nav_menu_object('Categories 2 Menu');
			$widget_menu_5_for_del = wp_get_nav_menu_object('Categories Menu');

			if ($all_pages_menu_for_del) {
				wp_delete_nav_menu('Main menu');
			}
			if ($top_menu_for_del) {
				wp_delete_nav_menu('Top menu');
			}
			if ($footer_menu_for_del) {
				wp_delete_nav_menu('Footer Menu');
			}
			if ($widget_menu_1_for_del) {
				wp_delete_nav_menu('Information');
			}
			if ($widget_menu_2_for_del) {
				wp_delete_nav_menu('Extras');
			}
			if ($widget_menu_3_for_del) {
				wp_delete_nav_menu('The Shop');
			}
			if ( $widget_menu_4_for_del ) {
				wp_delete_nav_menu( $widget_menu_4_for_del );
			}
			if ( $widget_menu_5_for_del ) {
				wp_delete_nav_menu( $widget_menu_5_for_del );
			}

			$this->delete_combo_products_depending_on_demo( $demo_name );

			// Force all variations to be imported
			add_filter( 'wp_import_existing_post', function ( $post_exists, $post ) {
				if ( $post['post_type'] === 'product_variation' ) {
					return 0;
				}

				return $post_exists;
			}, 10, 2 );

			$this->importWPContent($this->getDemoLocation() . '/' . $demo_name . '/demo.xml');

			echo 'rigid_wp_xml_import_success ' . date(DATE_RFC2822) . '<br/>';

			// Get Widget menus IDs, so we can pass them to widget import, so we have proper options set
			$widget_menu_1 = wp_get_nav_menu_object('Information');
			$widget_menu_2 = wp_get_nav_menu_object('Extras');
			$widget_menu_3 = wp_get_nav_menu_object('The Shop');
			$widget_menu_4 = wp_get_nav_menu_object('Categories Menu');
			$widget_menu_5 = wp_get_nav_menu_object('Categories 2 Menu');

			$settings_imp_result = $this->importSettings($this->getDemoLocation() . '/' . $demo_name . '/demo.txt', $widget_menu_1, $widget_menu_2, $widget_menu_3, $widget_menu_4, $widget_menu_5);
			if (is_wp_error($settings_imp_result)) {
				echo 'rigid_settings_import_error';
				return false;
			}
			echo 'rigid_settings_import_success ' . date(DATE_RFC2822) . '<br/>';

			// Install woocommerce pages
			if (defined('RIGID_IS_WOOCOMMERCE') && RIGID_IS_WOOCOMMERCE) {
				$shop_page = get_page_by_title('The Shop');
				if(!is_null($shop_page)) {
					update_option( 'woocommerce_shop_page_id', $shop_page->ID );
				}
				$account_page = get_page_by_title('My account');
				if(!is_null($account_page)) {
					update_option( 'woocommerce_myaccount_page_id', $account_page->ID );
				}
			}

			// Set Wishlist page
			if (defined('RIGID_IS_WISHLIST') && RIGID_IS_WISHLIST) {

				/** @var WP_Post $wishlist_page */
				$wishlist_page = get_page_by_title( 'Wishlist' );
				if ( is_null( $wishlist_page ) ) {
					$wishlist_page = get_page_by_title( 'My Wishlist' );
				}

				if($wishlist_page instanceof WP_Post) {
					update_option( 'yith-wcwl-page-id', $wishlist_page->ID );
					update_option( 'yith_wcwl_wishlist_page_id', $wishlist_page->ID );
				}
			}

			global /** @var WP_Rewrite $wp_rewrite */
			$wp_rewrite;
			$wp_rewrite->set_permalink_structure('/%postname%/');
			$wp_rewrite->flush_rules();

			// Set menus
			$all_pages_menu = wp_get_nav_menu_object('Main menu');
			// Set WCFM demo
			if ( $demo_name === 'rigid0' ) {
				if ( defined( 'RIGID_IS_WOOCOMMERCE' ) && RIGID_IS_WOOCOMMERCE ) {
					// Set attributes to the correct swatches
					$product_attributes = wc_get_attribute_taxonomies();
					foreach ( $product_attributes as $attribute ) {
						switch ( $attribute->attribute_name ) {
							case 'brand':
							case 'size':
							case 'gift':
								wc_create_attribute( array(
									'id'           => $attribute->attribute_id,
									'name'         => $attribute->attribute_name,
									'slug'         => $attribute->attribute_name,
									'type'         => 'label',
									'has_archives' => 1
								) );
								break;
							case 'color':
								wc_create_attribute( array(
									'id'           => $attribute->attribute_id,
									'name'         => $attribute->attribute_name,
									'slug'         => $attribute->attribute_name,
									'type'         => 'color',
									'has_archives' => 1
								) );
								break;
							case 'material':
								wc_create_attribute( array(
									'id'           => $attribute->attribute_id,
									'name'         => $attribute->attribute_name,
									'slug'         => $attribute->attribute_name,
									'type'         => 'image',
									'has_archives' => 1
								) );
								break;
						}
					}

					// Set products with attributes visible in listings
					$products_with_visible_in_listing = array(
						'amazon-echo-pod-alexa-smart-speaker'                          => 'pa_color',
						'drone-dji-mavic-air-wh-12-0-mpx-with-gps'                     => 'pa_color',
						'long-sleeve-shirt-2'                                          => 'pa_size',
						'hisense-tv-4'                                                 => 'pa_smart-os',
						'hisense-65a8383f-4k-ultra-hd-led-smart-tv-vidaa-65-0″'        => 'pa_smart-os',
						'hisense-tv-5'                                                 => 'pa_smart-os',
						'hisense-40a5600f-led-smart-tv-vidaa-40-0'                     => 'pa_smart-os',
						'light-grey-camisole'                                          => 'pa_smart-os',
						'hisense-tv-3'                                                 => 'pa_gift',
						'turtle-neck-sweater'                                          => 'pa_gift',
						'samsung-galaxy-watch-active-3-r830z-gd-40-mm'                 => 'pa_brand',
						'montech-desktop-gaming-pc-i7-10840-32gb-ram-geforce-rtx-3090' => 'pa_brand',
					);
					foreach ( $products_with_visible_in_listing as $product_name => $attribute_name ) {
						$product_post_object = get_page_by_path( $product_name, 'OBJECT', 'product' );
						if ( $product_post_object !== null ) {
							$product = wc_get_product( $product_post_object );
							update_post_meta( $product->get_id(), rigid_get_visible_in_listings_meta_field_name( $attribute_name ), 1 );
						}
					}

					// Fix product categories order
					$product_categories = get_terms( array(
						'taxonomy' => 'product_cat',
						'slug' => array(
							'photo-and-video',
							'desktops',
							'gaming',
							'audio',
							'it-accessories',
							'laptops',
							'tv-video',
							'outdoors-and-sports',
							'smart-home',
							'smartphones',
							'tablets',
							'wearables',
						)
					) );
					foreach ( $product_categories as $category ) {
						switch ( $category->slug ) {
							case 'tv-video':
								update_term_meta( $category->term_id, 'order', 1 );
								break;
							case 'smartphones':
								update_term_meta( $category->term_id, 'order', 2 );
								break;
							case 'tablets':
								update_term_meta( $category->term_id, 'order', 3 );
								break;
							case 'laptops':
								update_term_meta( $category->term_id, 'order', 4 );
								break;
							case 'desktops':
								update_term_meta( $category->term_id, 'order', 5 );
								break;
							case 'photo-and-video':
								update_term_meta( $category->term_id, 'order', 6 );
								break;
							case 'smart-home':
								update_term_meta( $category->term_id, 'order', 7 );
								break;
							case 'wearables':
								update_term_meta( $category->term_id, 'order', 8 );
								break;
							case 'outdoors-and-sports':
								update_term_meta( $category->term_id, 'order', 9 );
								break;
							case 'uncategorized':
								update_term_meta( $category->term_id, 'order', 10 );
								break;
							case 'gaming':
								update_term_meta( $category->term_id, 'order', 11 );
								break;
							case 'it-accessories':
								update_term_meta( $category->term_id, 'order', 12 );
								break;
							case 'audio':
								update_term_meta( $category->term_id, 'order', 13 );
								break;
						}
					}

					// Import WCFM options
					if ( class_exists( 'WCFM' ) ) {
						/** @var WCFM $WCFM */
						global $WCFM;
						$store_manager_page = get_page_by_title( 'Store Manager' );
						if ( $store_manager_page instanceof WP_Post ) {
							$wcfm_page_options                                = get_option( 'wcfm_page_options', array() );
							$wcfm_page_options['wc_frontend_manager_page_id'] = $store_manager_page->ID;
							update_option( 'wcfm_page_options', $wcfm_page_options );
						}
						$this->importWCFMSettings( $this->getDemoLocation() . '/' . $demo_name . '/wcfm_options.txt' );

						$WCFM->wcfm_create_custom_css();
						if ( class_exists( 'WCFMvm' ) ) {
							/** @var WCFMvm $WCFMvm */
							global $WCFMvm;
							$WCFMvm->wcfmvm_create_membership_css();
						}

						if ( class_exists( 'WCFMmp_Settings' ) ) {
							$wcfmmp_settings = new WCFMmp_Settings();
							$wcfmmp_settings->wcfmmp_create_store_css();
						}
					}
				}

				if( isset( $all_pages_menu->term_id )) {
					$menu_items = wp_get_nav_menu_items( $all_pages_menu->term_id );
					foreach ( $menu_items as $item ) {
						if ( in_array( $item->title, array( 'All Store Departments' ) ) ) {
							update_post_meta( $item->ID, '_rigid-menu-item-is_megamenu', 'horizontal' );
							update_post_meta( $item->ID, '_rigid-menu-item-opened_home', '1' );
						}
						if ( in_array( $item->title, array( 'Shop Pages' ) ) ) {
							update_post_meta( $item->ID, '_rigid-menu-item-is_megamenu', 'active' );
						}

						// Horizontal Menu descriptions that contain banners
						if ( in_array( $item->title, array( 'Column 3', 'Column 6', 'Column 7', 'Column 8', 'Column 9', 'Column 10' ) ) ) {
							update_post_meta( $item->ID, '_rigid-menu-item-is_description', 'active' );
							update_post_meta( $item->ID, '_rigid-menu-item-is_megamenu', 'horizontal' );
						}

						// Vertical Menu descriptions that contain banners
						if ( in_array( $item->title, array( 'The banner' ) ) ) {
							update_post_meta( $item->ID, '_rigid-menu-item-is_description', 'active' );
							update_post_meta( $item->ID, '_rigid-menu-item-is_megamenu', 'active' );
						}

						// Menu labels settings: [navigation label] => array([menu label], [menu color])
						$menu_labels_settings = array(
							'Shop Pages' => array( 'VERTICAL MEGA', '#9fc936' )
						);
						// Now do the labels
						foreach ( $menu_labels_settings as $title => $setting ) {
							if ( $item->title === $title ) {
								update_post_meta( $item->ID, '_rigid-menu-item-custom_label', $setting[0] );
								update_post_meta( $item->ID, '_rigid-menu-item-label_color', $setting[1] );
							}
						}
						// Menu Icons
						$menu_icons_settings = array(
							'All Store Departments' => 'fas fa-layer-group',
							'TV & Video'            => 'fas fa-tv',
							'Smartphones'           => 'fas fa-mobile',
							'Cameras'               => 'fas fa-camera',
							'PC and peripherials'   => 'fas fa-laptop',
							'IT accessories'        => 'fas fa-mouse',
							'Tablets'               => 'fas fa-tablet-alt',
							'Gaming'                => 'fas fa-gamepad',
							'Smart Home'            => 'fas fa-microphone-alt',
							'Shop Pages'            => 'fas fa-store',
							'Shop Page Sidebar'     => 'fas fa-indent',
							'Shop Page no Sidebar'  => 'fas fa-align-justify',
							'Product Category'      => 'fas fa-clipboard',
							'Product Attribute'     => 'fas fa-icons',
							'Product Tag'           => 'fas fa-tag',
							'Product Search'        => 'fas fa-search',
							'Shop List View'        => 'fas fa-list-alt',
							'Load More Button'      => 'fas fa-spinner',
							'Standard Pagination'   => 'fas fa-copy',
							'Samsung'               => 'fas fa-arrow-circle-right',
							'Huawei'                => 'fas fa-arrow-circle-right',
							'Philips'               => 'fas fa-arrow-circle-right',
							'Product With Addons'   => 'fas fa-bell',
							'Combo Product'         => 'fas fa-exclamation-triangle',
							'Limited Time Deal'     => 'fas fa-stopwatch',
							'Vendor Stores'         => 'fas fa-store',
							'Single Vendor Store'   => 'fas fa-store-alt',
							'Vendor Registration'   => 'fas fa-user-tag',
							'Vendor Membership'     => 'fas fa-user-plus',
							'Vendor Dashboard'      => 'fas fa-id-card',
						);
						// Now do the Icons
						foreach ( $menu_icons_settings as $title => $icon ) {
							if ( $item->title === $title ) {
								update_post_meta( $item->ID, '_rigid-menu-item-icon', $icon );
							}
						}
						// Menu images
						$menu_images_settings = array(
							'Red'   => 'red_dot_menu',
							'Blue'  => 'blue_dot_menu',
							'Green' => 'green_dot_menu',
							'Black' => 'black_dot_menu',
						);
						// Now do the images
						foreach ( $menu_images_settings as $title => $image ) {
							if ( $item->title === $title ) {
								$image_id = $this->get_attachment_id_by_slug( $image );
								update_post_meta( $item->ID, '_rigid-menu-item-image', $image_id );
							}
						}
					}
				}
			}
			// Set fashion demo
			if ($demo_name === 'rigid1' && isset($all_pages_menu->term_id)) {
				if (defined('RIGID_IS_WOOCOMMERCE') && RIGID_IS_WOOCOMMERCE) {
					// Set attributes to the correct swatches
					$product_attributes = wc_get_attribute_taxonomies();
					foreach ( $product_attributes as $attribute ) {
						switch ( $attribute->attribute_name ) {
							case 'brand':
							case 'size':
								wc_create_attribute( array(
									'id'   => $attribute->attribute_id,
									'name' => $attribute->attribute_name,
									'slug' => $attribute->attribute_name,
									'type' => 'label',
									'has_archives' => 1
								) );
								break;
							case 'color':
								wc_create_attribute( array(
									'id'   => $attribute->attribute_id,
									'name' => $attribute->attribute_name,
									'slug' => $attribute->attribute_name,
									'type' => 'color',
									'has_archives' => 1
								) );
								break;
							case 'material':
								wc_create_attribute( array(
									'id'   => $attribute->attribute_id,
									'name' => $attribute->attribute_name,
									'slug' => $attribute->attribute_name,
									'type' => 'image',
									'has_archives' => 1
								) );
								break;
						}
					}
				}

				$menu_items = wp_get_nav_menu_items($all_pages_menu->term_id);

				foreach ($menu_items as $item) {
					if (in_array($item->title, array('Home', 'The Shop', 'Collections'))) {
						update_post_meta($item->ID, '_rigid-menu-item-is_megamenu', 'active');
					}

					// Menu descriptions that contain banners
					if(in_array($item->title, array('home text 1', 'home text 2', 'home menu banner', 'Category Options', 'Copyright', 'Product Options', 'Shop Options', 'Port_pic1', 'Port_pic2', 'Port_pic3', 'Port_pic4', 'Port_pic5', 'Shop_pic1'))) {
						update_post_meta($item->ID, '_rigid-menu-item-is_description', 'active');
						update_post_meta($item->ID, '_rigid-menu-item-is_megamenu', 'active');
					}

					// Menu labels settings: [navigation label] => array([menu label], [menu color])
					$menu_labels_settings = array(
						'The Shop' => array('MEGA MENU', ''),
						'Collections' => array('a.k.a. PORTFOLIO', ''),
						'Combo Product 1' => array('HOT', '#f28e37'),
						'Combo Product 2' => array('HOT', '#f28e37'),
						'Product With Addons' => array('HOT', '#99c43c'),
						'Coming Soon' => array('BLANK PAGE', ''),
					);
					// Now do the labels
					foreach ($menu_labels_settings as $title => $setting) {
						if ( $item->title === $title ) {
							update_post_meta( $item->ID, '_rigid-menu-item-custom_label', $setting[0] );
							update_post_meta( $item->ID, '_rigid-menu-item-label_color', $setting[1] );
						}
					}
					// Menu Icons
					$menu_icons_settings = array(
						'Pages' => 'fas fa-file-alt',
						'WooCommerce Features' => 'fas fa-shopping-bag',
						'Visual Composer<sup>INCLUDED</sup>' => 'fas fa-cubes',
						'Revolution Slider<sup>INCLUDED</sup>' => 'fas fa-sync',
						'3-rd Party Plugin Integrations' => 'fas fa-plug',
					);
					// Now do the Icons
					foreach ($menu_icons_settings as $title => $icon) {
						if ( $item->title === $title ) {
							update_post_meta( $item->ID, '_rigid-menu-item-icon', $icon );
						}
					}
				}
			}

			// Set WC Marketplace demo
			if ($demo_name === 'rigid2' && isset($all_pages_menu->term_id)) {
				if (defined('RIGID_IS_WOOCOMMERCE') && RIGID_IS_WOOCOMMERCE) {
					// Set attributes to the correct swatches
					$product_attributes = wc_get_attribute_taxonomies();
					foreach ( $product_attributes as $attribute ) {
						switch ( $attribute->attribute_name ) {
							case 'size':
								wc_create_attribute( array(
									'id'   => $attribute->attribute_id,
									'name' => $attribute->attribute_name,
									'slug' => $attribute->attribute_name,
									'type' => 'label',
									'has_archives' => 1
								) );
								break;
							case 'color':
								wc_create_attribute( array(
									'id'   => $attribute->attribute_id,
									'name' => $attribute->attribute_name,
									'slug' => $attribute->attribute_name,
									'type' => 'color',
									'has_archives' => 1
								) );
								break;
						}
					}
				}

				$menu_items = wp_get_nav_menu_items($all_pages_menu->term_id);

				foreach ($menu_items as $item) {
					if (in_array($item->title, array('Market', 'Vendors'))) {
						update_post_meta($item->ID, '_rigid-menu-item-is_megamenu', 'active');
					}

					// Menu descriptions
					if(in_array($item->title, array('Category Options', 'Pic 11', 'Shop_pic1', 'Product Options'))) {
						update_post_meta($item->ID, '_rigid-menu-item-is_description', 'active');
						update_post_meta($item->ID, '_rigid-menu-item-is_megamenu', 'active');
					}
				}

				// Set blog page
				if ( get_page_by_title( 'Stories' ) ) {
					$blog_page = get_page_by_title( 'Stories' );
					if ( $blog_page instanceof WP_Post ) {
						update_option( 'show_on_front', 'page' );
						update_option( 'page_for_posts', $blog_page->ID );
					}
				}

				// Enable user registrations
				update_option( 'users_can_register', 1 );

				// Set MVX vendor dashboard and registration pages
				if ( function_exists( 'update_mvx_vendor_settings' ) ) {
					$vendor_dashboard_page = get_page_by_title( 'Vendor Dashboard' );
					if ( $vendor_dashboard_page instanceof WP_Post ) {
						update_mvx_vendor_settings( 'dc_vendor', $vendor_dashboard_page->ID, 'vendor', 'general' );
					}
					$vendor_registration_page = get_page_by_title( 'Vendor Registration' );
					if ( $vendor_dashboard_page instanceof WP_Post ) {
						update_mvx_vendor_settings( 'vendor_registration', $vendor_registration_page->ID, 'vendor', 'general' );
					}
				}
			}

			// Set sneakers demo
			if ($demo_name === 'rigid3' && isset($all_pages_menu->term_id)) {
				$menu_items = wp_get_nav_menu_items($all_pages_menu->term_id);

				foreach ($menu_items as $item) {
					if (in_array($item->title, array('Mens', 'Womens'))) {
						update_post_meta($item->ID, '_rigid-menu-item-is_megamenu', 'active');
					}

					// Menu descriptions
					if(in_array($item->title, array('Category Options', 'W Category Options', 'Shoe 1', 'Shoe 2', 'Shoe 3', 'Shoe 4', 'Shoe 6', 'W Shoe 1', 'W Shoe 2', 'W Shoe 3', 'W Shoe 5', 'W Shoe 4', 'W Shoe 6'))) {
						update_post_meta($item->ID, '_rigid-menu-item-is_description', 'active');
						update_post_meta($item->ID, '_rigid-menu-item-is_megamenu', 'active');
					}
				}
			}

			// Set vape demo
			if ( $demo_name === 'rigid4' ) {
				if ( defined( 'RIGID_IS_WOOCOMMERCE' ) && RIGID_IS_WOOCOMMERCE ) {
					// Set attributes to the correct swatches
					$product_attributes = wc_get_attribute_taxonomies();
					foreach ( $product_attributes as $attribute ) {
						switch ( $attribute->attribute_name ) {
							case 'color':
								wc_create_attribute( array(
									'id'           => $attribute->attribute_id,
									'name'         => $attribute->attribute_name,
									'slug'         => $attribute->attribute_name,
									'type'         => 'color',
									'has_archives' => 1
								) );
								break;
						}
					}
				}

				if ( isset( $all_pages_menu->term_id ) ) {
					$locations['mobile']   = $all_pages_menu->term_id;
					$locations['tertiary'] = $all_pages_menu->term_id;
					set_theme_mod( 'nav_menu_locations', $locations );
				}

				// Setup widget menus specific to demo 4
				$extras_menu = wp_get_nav_menu_object( 'Extras' );
				$info_menu   = wp_get_nav_menu_object( 'Information' );
				if ( isset( $info_menu->term_id ) && isset( $extras_menu->term_id ) ) {
					$widget_nav_menu                = get_option( 'widget_nav_menu' );
					$widget_nav_menu[2]['nav_menu'] = $info_menu->term_id;
					$widget_nav_menu[4]['nav_menu'] = $extras_menu->term_id;
					update_option( 'widget_nav_menu', $widget_nav_menu );
				}
			}

			$top_menu = wp_get_nav_menu_object('Top menu');
			$footer_menu = wp_get_nav_menu_object('Footer Menu');

			$locations = get_theme_mod('nav_menu_locations');
			if ($all_pages_menu) {
				$locations['primary'] = $all_pages_menu->term_id;
			}
			if ($top_menu) {
				$locations['secondary'] = $top_menu->term_id;
			}
			if ($footer_menu) {
				$locations['tertiary'] = $footer_menu->term_id;
			}
			set_theme_mod('nav_menu_locations', $locations);

			// Set home and blog pages
			$front_page = get_page_by_path('Home');

			if(get_page_by_title('The Blog')) {
				$blog_page = get_page_by_title( 'The Blog' );
			} else {
				$blog_page = get_page_by_title( 'News' );
			}

			if ($front_page) {
				update_option('show_on_front', 'page');
				update_option('page_on_front', $front_page->ID);
			}

			if ($blog_page) {
				update_option('show_on_front', 'page');
				update_option('page_for_posts', $blog_page->ID);
			}

			// import rev sliders
			if (RIGID_IS_REVOLUTION) {
				$this->importRevSliders($demo_name);
			}

			return true;
		}

		private function delete_combo_products_depending_on_demo( string $demo_name ) {
			if ( defined( 'RIGID_IS_WOOCOMMERCE' ) && RIGID_IS_WOOCOMMERCE ) {
				$demo_combo_product_pair = array(
					'rigid0' => 'smartphone-music-combo',
					'rigid1' => 'boots-and-purse-combo',
				);
				if ( ! empty( $demo_combo_product_pair[ $demo_name ] ) ) {
					$combo_slug          = $demo_combo_product_pair[ $demo_name ];
					$product_post_object = get_page_by_path( $combo_slug, 'OBJECT', 'product' );
					if ( $product_post_object !== null ) {
						$product = wc_get_product( $product_post_object );
						if ( get_class( $product ) === 'WC_Product_Combo' ) {
							$product->delete( true );
						}
					}
				}
			}
		}

	}

}